package mhtree.benchmarking;

import messif.objects.LocalAbstractObject;
import messif.objects.impl.ObjectFloatVectorNeuralNetworkL2;
import messif.objects.util.AbstractObjectList;
import messif.objects.util.AbstractStreamObjectIterator;
import messif.objects.util.StreamGenericAbstractObjectIterator;

import java.io.IOException;
import java.util.List;
import java.util.function.Consumer;
import java.util.logging.Level;
import java.util.logging.Logger;

public class BenchmarkConfig {
    private List<MHTreeConfig> mhTreeConfigs;
    private List<String> datasetPaths;
    private List<Consumer<MHTreeConfig>> benchmarkFunctions;

    public BenchmarkConfig(List<MHTreeConfig> mhTreeConfigs, List<String> datasetPaths, List<Consumer<MHTreeConfig>> benchmarkFunctions) {
        this.mhTreeConfigs = mhTreeConfigs;
        this.datasetPaths = datasetPaths;
        this.benchmarkFunctions = benchmarkFunctions;
    }

    public void execute() {
        for (String datasetPath : datasetPaths) {
            AbstractStreamObjectIterator<ObjectFloatVectorNeuralNetworkL2> objectIterator;

            try {
                objectIterator = new StreamGenericAbstractObjectIterator<>(ObjectFloatVectorNeuralNetworkL2.class, datasetPath);
            } catch (IOException e) {
                Logger.getLogger("MH-Tree Benchmark").log(Level.SEVERE, null, e);
                return;
            }

            List<LocalAbstractObject> objects = new AbstractObjectList<>(objectIterator);

            for (MHTreeConfig mhTreeConfig : mhTreeConfigs) {
                mhTreeConfig.setObjects(objects);

                for (int i = 0; i < benchmarkFunctions.size(); i++) {
                    System.out.println("~Running" + mhTreeConfig + " on " + i + "-th function");
                    benchmarkFunctions.get(i).accept(mhTreeConfig);
                }
            }
        }
    }
}
